// This is a simple LDAP authentication provider for CA Live API Creator.
// Each user has an attribute named mail, and some users have an attribute named telephoneNumber.
//
// To adapt this to your specific LDAP environment, you will have to change the code to fit
// your schema, e.g. what attribures are relevant, how you figure out which roles a user has, etc...
//
// Adapted from SimpleLDAPAuthProvider for Active Directory.
// Uses a fixed attribute to check for corresponding role in Live API Creator

function ActiveDirectoryAuthProvider () {
var result = {};
var configSetup = {};
// This gets called first to pass in the required LDAP configuration values
result.configure = function configure(myConfig) {
               configSetup.serverName = myConfig.serverName || "ldap://10.0.2.100:389";
               configSetup.keyLifetimeMinutes = myConfig.keyLifetimeMinutes || 60;
};
// This gets called to validate the user payload against LDAP service
result.authenticate = function authenticate(payload) {
java.lang.System.out.println("Authenticating with LDAP: " + payload.username);
var authResponse = {
roleNames: [], //Hardcode in "roleNames: {role}," here if you wish to have open auth for anyone who can authenticate
userIdentifier: payload.username,
keyLifetimeSeconds: configSetup.keyLifetimeMinutes,
userData: {},
// We hard-code userInfo here, but you can add information that will be returned
// to the authenticator in the response, along with the auth token.
userInfo: {typeOfPerson: 'Active Directory User'},
// If you are writing an auth provider for use with the *admin* project,
// then you will need to include the accountIdent in the userInfo, e.g.:
// userInfo: {typeOfPerson: 'Cool', accountIdent: 1000},
// If you know when and from where this user last logged in, you can
// optionally return it here.
lastLogin : {
datetime: null,
ipAddress : null
}
};
// Set up the JNDI environment
var Hashtable = Java.type("java.util.Hashtable");
var env = new Hashtable();
env.put("java.naming.factory.initial", "com.sun.jndi.ldap.LdapCtxFactory");
env.put("java.naming.provider.url", configSetup.serverName);
var userCN = "cn=" + payload.username + ",ou=company,dc=democompany,dc=local";
env.put("java.naming.security.authentication", "simple");
env.put("java.naming.security.principal", userCN);
env.put("java.naming.security.credentials", payload.password);
env.put("java.naming.referral", "follow");
var InitialDirContext = Java.type("javax.naming.directory.InitialDirContext");
try {
// First, can we connect? If not, then either the user is unknown,
// or the password is wrong, and this will throw an exception.
var ctx = null;

try {
ctx = new InitialDirContext(env);
}
catch(e) {
               return {
                              errorMessage: "Unable to authenticate with LDAP server: " + e.getMessage()
               }
}
java.lang.System.out.println("Authenticating Success: " + payload.username);
// Next, we retrieve the mail and phone attributes of the user, if present
var attrs = ctx.getAttributes(userCN, ["mail", "telephoneNumber"]);
var attrsEnum = attrs.getAll();
while (attrsEnum.hasMore()) {
               var attrib = attrsEnum.next();
               authResponse.userData[attrib.getID()] = attrib.get().toString();
}

// If there are multi-valued attributes, this is how to retrieve them
attrs = ctx.getAttributes(userCN, ["departmentNumber"]); //specify the attribute in AD for API user role, here it is "departmentNumber"
attrsEnum = attrs.getAll();
if (attrsEnum.hasMore()) {
			var mvAttr = attrsEnum.next();
            var attrEnum = mvAttr.getAll();
			while (attrEnum.hasMore()) {
                              var roleName = attrEnum.next();
                              java.lang.System.out.println("Getting Roles: " +roleName);
               // If the attribute roleName is equal to below then push the role to the auth response
               if (roleName.equals("LAC-admins")) {
               authResponse.roleNames.push('admins'); //ensure that this corresponding rolename is created in LAC
               }
		}
	}
}
 
catch(e) {
return {
errorMessage: e.getMessage()
}
}

java.lang.System.out.println(JSON.stringify(authResponse));
return authResponse;
};
// getLoginInfo gets called to create the logon dialog - you can change this if your
// authentication method does not use username/password, or if you don't like the names,
// but of course you'll have to change the authenticate function correspondingly.
result.getLoginInfo = function getLoginInfo() {
return {
fields: [
{
                name: "username",
                display: "User name",
                description: "Enter your Username, e.g. einstein",
                type: "text",
                length: 40,
                helpURL: ""
 },
 {
                name: "password",
                display: "Password",
                description: "Enter your password, e.g. password",
                type: "password",
                               length: 40,
                               helpURL: ""
                }
 ],
 links : []
                              };
};
// This function is called to determine what parameters can be configured in this auth provider.
// The values are stored in the admin database when the user saves the auth provider.
result.getConfigInfo = function getConfigInfo() {
               return {
                              current : {
                                "serverName": configSetup.serverName,
                                "keyLifetimeMinutes" : configSetup.keyLifetimeMinutes
                              },
                              fields : [
                              {
                                name: "serverName",
                                display: "LDAP Server Name",
                                type: "text",
                                length: 40,
                                helpURL: ""
                              },
                              {
                                name: "keyLifetimeMinutes",
                                display: "API Key Lifetime (Minutes)",
                                type: "number",
                                length: 8,
                                helpURL: ""
                              }
                              ],
                              links: []
               };
};
return result;
}